/*******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2004-2007 Freescale Semiconductor, Inc.
* (c) Copyright 2001-2004 Motorola, Inc.
* ALL RIGHTS RESERVED.
*
********************************************************************************
*
* FILE NAME: main.c   
*
* DESCRIPTION: Sample application which shows how to simply toggle a GPIO LED
*              pin and also how to handle interrupts from GPIO. Upon each GPIO 
*              interrupt (coming from pushbuttons [SW4, SW5]) the red or green LED toggles. 
*              Another green LED toggles periodically in the main application loop.
*
* TARGET: MC56F8xxx devices
*
*******************************************************************************/

#include "safety_config.h"
#include "safety_test_items.h"

#include "iec60730b.h"

/* board-specific LEDs and buttons */
#include "../board.h"

#include "qs.h"
#include "sys.h"
#include "intc.h"
#include "gpio.h"
#include "cop.h"
#include "adc12.h"
#include "qs.h"
#include "qtimer.h"
#include "pit.h"
#include "xbar.h"
#include "sci.h"
#include "pmc.h"
#include "freemaster.h"

/* forward prototypes */
void device_init(void);

#pragma define_section safety_ram ".safety_ram.data" RW
#pragma section safety_ram begin
safety_common_t g_sSafetyCommon;
fs_clock_test_t g_sSafetyClockTest;

wd_test_t g_sSafetyWdTest;
unsigned long ui32RamTestArray[10];
#pragma section safety_ram end

extern UInt32 _safety_ram; /* from Linker command file */
extern UInt32 _end_safety_ram; /* from Linker command file */

unsigned long ui32RamBackupArray[10];


typedef struct {
	unsigned long start_addr;
	unsigned long mem_length;
	unsigned short mem_type;
	unsigned short poly;
	unsigned short crc16;
} CRC16_record;  

extern __pmem CRC16_record g_crc_linker;
extern UInt16 _pc_test_flag;   /* from Linker configuration file */
extern UInt32 start_text;
extern UInt32 end_text;
flash_runtime_test_parameters_t g_sFlashCrc;
flash_configuration_parameters_t g_sFlashConfig;

volatile UInt32 ui32ResetSource;
extern uint16_t ui16ResetExt;

uint8_t i;
volatile uint16_t exception_counter = 0;
volatile uint16_t background_counter = 0;
volatile uint16_t clock_test_condition = 0;

ram_test_t      g_sSafetyRamTest;
unsigned short ui16BlockSize = 0x4;
unsigned short ui16RamTestRange_11 = 0x11;
unsigned short ui16RamTestRange_12 = 0x12;
unsigned short ui16RamTestRange_13 = 0x13;

void main(void)
{	
	UInt32 pui32SafetyRamSectionStart = (UInt32)&_safety_ram;
	UInt32 pui32SafetyRamSectionEnd = (UInt32)&_end_safety_ram;
	UInt32 *ui32SafetyRamStart = (UInt32 *)pui32SafetyRamSectionStart;
	UInt32 *ui32SafetyRamEnd = (UInt32 *)pui32SafetyRamSectionEnd;	
	
    /* enable clock to CRC module */
	ioctl(SIM, SYS_INIT, SIM_PCE2_INIT);
	/* initialize SYS, COP and pins */

	g_sSafetyCommon.safetyErrors = 0;
	
	/* Watchdog - COP configuration */
	ioctl(COP,COP_INIT,NULL); /* 100ms COP timeout */
	/*****************/
	/* Watchdog test */
	/*****************/
	ui32ResetSource = SafetyWatchdogTest(&g_sSafetyWdTest);

	/*****************/
    /* Stack test    */
	/*****************/
    SafetyStackTestInit();
    SafetyStackTest(&g_sSafetyCommon);
    
    /*****************************************************************/
    ioctl(COP, COP_CLEAR_COUNTER, NULL); /* COP refresh **************/
    /*****************************************************************/
    
    /*****************/
    /* DIO test      */
    /*****************/
    for (i=0; dio_safety_test_items[i]!= 0; i++)
    {
		SafetyDigitalOutputTest(&g_sSafetyCommon,dio_safety_test_items[i]);
		SafetyDigitalInputOutput_ShortSupplyTest(&g_sSafetyCommon,dio_safety_test_items[i], DIO_SHORT_TO_GND_TEST);
		SafetyDigitalInputOutput_ShortSupplyTest(&g_sSafetyCommon,dio_safety_test_items[i], DIO_SHORT_TO_VDD_TEST);
    }
    	
    /*****************************************************************/
    ioctl(COP, COP_CLEAR_COUNTER, NULL); /* COP refresh **************/
    /*****************************************************************/

    /*****************/
    /* DIO test      */
    /*****************/    
    SafetyDigitalInputOutput_ShortAdjTest(&g_sSafetyCommon, dio_safety_test_items[0],dio_safety_test_items[1], LOGICAL_ONE); 
    SafetyDigitalInputOutput_ShortAdjTest(&g_sSafetyCommon, dio_safety_test_items[0],dio_safety_test_items[1], LOGICAL_ZERO); 
    	
    /*****************************************************************/
    ioctl(COP, COP_CLEAR_COUNTER, NULL); /* COP refresh **************/
    /*****************************************************************/
 
    /*****************/
    /* Flash test    */
    /*****************/   
    g_sFlashConfig.checksum16 = g_crc_linker.crc16;
    g_sFlashConfig.startAddress = (UInt32)&start_text;
    g_sFlashConfig.size = (UInt32)&end_text - (UInt32)&start_text;
    g_sFlashConfig.startConditionSeed = 0;
    g_sFlashConfig.blockSize = FLASH_TEST_BLOCK_SIZE;
    SafetyFlashAfterResetTest(&g_sSafetyCommon, &g_sFlashConfig);
    

	 
    /*****************************************************************/
    ioctl(COP, COP_CLEAR_COUNTER, NULL); /* COP refresh **************/
    /*****************************************************************/
    
    /*****************/
    /* RAM test    */
    /*****************/     
    SafetyRamTestInit(&g_sSafetyRamTest, ui32SafetyRamStart, ui32SafetyRamEnd);  
    SafetyRamAfterResetTest(&g_sSafetyCommon, &g_sSafetyRamTest);
	
    /*****************************************************************/
    ioctl(COP, COP_CLEAR_COUNTER, NULL); /* COP refresh **************/
    /*****************************************************************/

    /*****************/
    /* PC test    */
    /*****************/     
    FS_PC_Test(FS_PC_object_1, FS_PC_subroutine, (unsigned short *)&_pc_test_flag);
    FS_PC_Test(FS_PC_object_2, FS_PC_subroutine, (unsigned short *)&_pc_test_flag);
    
    /*****************************************************************/
    ioctl(COP, COP_CLEAR_COUNTER, NULL); /* COP refresh **************/
    /*****************************************************************/

    /*****************/
    /* CPU test      */
    /*****************/ 
    g_sSafetyCommon.safetyErrors |= FS_CPU_DataRegisters();
    g_sSafetyCommon.safetyErrors |= FS_CPU_PointerRegisters();
    g_sSafetyCommon.safetyErrors |= FS_CPU_ShadowRegistersE();
    g_sSafetyCommon.safetyErrors |= FS_CPU_ShadowRegistersEX();
    g_sSafetyCommon.safetyErrors |= FS_CPU_StatusRegister();
    g_sSafetyCommon.safetyErrors |= FS_CPU_StackPointer();
    
    /*****************************************************************/
    ioctl(COP, COP_CLEAR_COUNTER, NULL); /* COP refresh **************/
    /*****************************************************************/

    /*****************/
    /* CPU test      */
    /*****************/     
    g_sSafetyCommon.safetyErrors |= FS_CPU_LoopRegisters();

    device_init();
    archEnableInt();
    /* ADC init */
    ioctl(PMC, PMC_INIT, NULL);       /* enable internal reference for ANA 7 channel */
    ioctl(GPIO_A, GPIO_INIT, NULL);   /* set GPIOA4. GPIOA5 as ANA4, ANA5 inputs */
    ioctl(ADC, ADC_INIT, NULL);       /* initialisation of ADC module */
    SafetyAnalogTestInitialization(); /* initialisation of ADC test */
     	
    /*****************************************************************/
    ioctl(COP, COP_CLEAR_COUNTER, NULL); /* COP refresh **************/
    /*****************************************************************/

    /*****************/
    /* ADC test      */
    /*****************/     
    for (i=0;i<6;i++)
    {
        SafetyAnalogTest(&g_sSafetyCommon);
    }
    
    /*****************/
    /* Clock test    */
    /*****************/ 
    SafetyClockTestInit(&g_sSafetyCommon, &g_sSafetyClockTest);

	ioctl(XBAR_A, XBAR_A_INIT, NULL); /* QTA0_input PIT0_SYNC_OUT, QTA1_input PIT0_SYNC_OUT */
	ioctl(PIT_0, PIT_INIT, NULL);    /* RC oscilator 200kHz, roll over time 10us */
    ioctl(QTIMER_A0, QT_INIT, NULL); /* 50 MHz */
    
    ioctl(GPIO_D, GPIO_INIT, NULL);
    ioctl(SCI_2,SCI_INIT,NULL);
    FMSTR_Init();

    /* PIT_1 configuration for interrupt */
    ArchIO.Pit1.ctrl |= PIT_CTRL_CLKSEL_CLOCK3;
    ArchIO.Pit1.ctrl |= PIT_CTRL_PRIE;
    ArchIO.Pit1.mod  |= 5000;
    ArchIO.Pit1.ctrl |= PIT_CTRL_CNT_EN;
    
    while(1)
    {
    	background_counter ++;
    	
    	g_sSafetyCommon.safetyErrors |= FS_CPU_PointerRegisters();
    	g_sSafetyCommon.safetyErrors |= FS_CPU_ShadowRegistersE();
    	g_sSafetyCommon.safetyErrors |= FS_CPU_ShadowRegistersEX();
    	   	
    	SafetyFlashRuntimeTest(&g_sSafetyCommon, &g_sFlashCrc, &g_sFlashConfig);
    	
    	FMSTR_Poll();

        /*****************/
        /* Clock test    */
        /*****************/ 
    	if (exception_counter >50)
    		clock_test_condition |=1;
    	
    	if(clock_test_condition)
            SafetyClockTestCheck(&g_sSafetyCommon, &g_sSafetyClockTest);  
    }
}


/******************************** interrupt ******************************/
#pragma interrupt saveall
void Pit1Isr(void)
{
	exception_counter ++;

    ioctl(COP, COP_CLEAR_COUNTER, NULL); /* COP refresh **************/   	

	SafetyRamRuntimeTest(&g_sSafetyCommon, &g_sSafetyRamTest);	

	/* clear the roll-over flag */
	ArchIO.Pit1.ctrl &= ~PIT_CTRL_PRF;	
}
#pragma interrupt off


/********************************* device_init *****************************/
#define ADC_CTRL1_START0_MASK (1<<13)
void device_init(void)
{
	static uint16_t adc_ctrl1 = 0;
	static uint16_t adc_ctrl2 = 0;
	static uint16_t restul0=0xFFFF;
	static uint16_t restul1=0xFFFF;
	static uint16_t restul2=0xFFFF;
	FS_ADC_CYCLIC_Type * adc_base = (FS_ADC_CYCLIC_Type*)ADC_BASE;
	
    ioctl(SYS, SYS_INIT, NULL);
  
    ioctl(INTC,INTC_INIT, NULL);
	ioctl(ADC, ADC_INIT, NULL);
	
    /* Note: This code is targeted to all 56F8xxx-based evaluation boards.
       So we need to check what GPIO instances are actually implemented */
    #ifdef GPIO_A
    ioctl(GPIO_A, GPIO_INIT, NULL);
    #endif
    #ifdef GPIO_B
    ioctl(GPIO_B, GPIO_INIT, NULL);
    #endif
    #ifdef GPIO_C
    ioctl(GPIO_C, GPIO_INIT, NULL);
    #endif
    #ifdef GPIO_D
    ioctl(GPIO_D, GPIO_INIT, NULL);
    #endif
    #ifdef GPIO_E
    ioctl(GPIO_E, GPIO_INIT, NULL);
    #endif
    #ifdef GPIO_F
    ioctl(GPIO_F, GPIO_INIT, NULL);
    #endif    
}
