/*
 * Copyright (c) 2016, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _SAFETY_CONFIG_H_
#define _SAFETY_CONFIG_H_

#include <stdint.h>
typedef unsigned long long uint64_t;
typedef signed long long int64_t;

#include "MC56F83789.h"
#include "iec60730b.h"
#include "project_setup_mc56f83789.h"
#include "safety_dsc.h"
#include "fsl_pit.h"

/*******************************************************************************
 * Definitions
 ******************************************************************************/
/* This macro enables infinity while loop in SafetyErrorHandling() function */
#define SAFETY_ERROR_ACTION 0

/* TEST SWITCHES - for debugging it is better to turn the flash test and watchdog OFF */
#define ADC_TEST_ENABLED   0
#define CLOCK_TEST_ENABLED 1
#define DIO_TEST_ENABLED   1
#define FLASH_TEST_ENABLED 1
#define RAM_TEST_ENABLED   1 
#define PC_TEST_ENABLED    1

#define FMSTR_SERIAL_ENABLE 0

#define FAST_IRC_CLOCK   8000000U /*Frequency in  Hz*/
#define PIT0_CLOCK       FAST_IRC_CLOCK
#define PIT1_CLOCK       FAST_IRC_CLOCK
#define PIT0_CLOCK_DIV   kPIT_PrescalerDivBy128
#define PIT0_RELOAD      3052
/* CLOCK test */
#define QTIMER_PRIMARY_CLK         100000000U /*Frequency in Hz*/
#define QTIMER_SECONDARY_CLK         PIT1_CLOCK
#define CLOCK_TEST_TOLERANCE         20  /* % */

/* WDOG test */
#if(1)
    #define WATCHDOG_TEST_ON
#endif

#if(1)
    #define WATCHDOG_ENABLED   
#endif

#define COP_TIMEOUT_IN_MICRO_SECONDS       100000 
#define COP_TIMEOUT_IN_MILI_SECONDS  (COP_TIMEOUT_IN_MICRO_SECONDS/1000)
#define PIT_32BIT_COUNTER  0
#define PIT1_RELOAD   0xFFFF
#define PIT1_CLOCK_DIV   kPIT_PrescalerDivBy64
#define PIT1_FREQ        PIT1_CLOCK>>PIT1_CLOCK_DIV

#define ENDLESS_LOOP_ENABLE           0    /* set 1 or 0 */
#define WATCHDOG_RESETS_LIMIT         1000 
#define WD_TEST_TOLERANCE             20    /* % */ 

/* GPIO macros */
#define  GPIO_BASE              GPIOG_BASE   //0xE260
#define TESTED_PIN_1            1
#define TESTED_PIN_2            2

#define PIN_DIRECTION_IN  0
#define PIN_DIRECTION_OUT 1

#define PIN_PULL_DISABLE  0
#define PIN_PULL_DOWN     1
#define PIN_PULL_UP       2

#define LOGICAL_ONE   1
#define LOGICAL_ZERO  0

/* DIO test */
#define DIO_WAIT_CYCLE 75

#define DIO_BACKUP_ENABLE  1
#define DIO_BACKUP_DISABLE 0
#define DIO_BACKUP DIO_BACKUP_ENABLE 

#define DIO_SHORT_TO_GND_TEST   1
#define DIO_SHORT_TO_VDD_TEST   0
 


///* SERIAL macros */
//#define USE_UART 1 /* 1 = UART, 0 = LPUART */
//#define APPLICATION_SERIAL_BASE USART0      
//#define SERIAL_BAUD_RATE      9600
//#define SERIAL_CLOCK          30e6
//#define UART_TX_PIN           49 // P1_17
//#define UART_RX_PIN           48 // P1_16

/* FLASH test */
#define HW_FLASH_TEST 1 /* HW=1, SW=0 */
#define FLASH_TEST_BLOCK_SIZE     0x20
#define FLASH_TEST_CONDITION_SEED 0x0000 
#define FLASH_TEST_START_ADDRESS  0x200
#define FLASH_TEST_END_ADDRESS    0x500

/* RAM test */
#define RAM_TEST_BLOCK_SIZE           0x2 /* size of block for runtime testing */

/* STACK test */
#define STACK_TEST_PATTERN 0x77777777
/* STACK_TEST_BLOCK_SIZE defined in safety_dsc.c */

/* ADC test */
#define IEC60730B_CFG_AIO_CHANNELS_CNT       3 
#define IEC60730B_CFG_AIO_CHANNELS_INIT {4,5,7}  /* ADC Channels to scan*/
#define IEC60730B_CFG_AIO_SAMPLES_INIT  {0,1,1}   /* ADC SAMPLES from CLISTx registers */
/* VDD */
/* GND */
/* Internal Voltage Reference */
#if(0)
#define IEC60730B_CFG_AIO_CHANNELS_LIMITS_INIT \
{\
  {3900, 4100},\
  {   0,   30},\
  { 900, 1200}\
}
#endif
#define IEC60730B_CFG_AIO_CHANNELS_LIMITS_INIT \
{\
  {3800, 4100},\
  {   0,   200},\
  { 800, 1300}\
}


#endif /* _SAFETY_CONFIG_H_ */
