/*
 * Copyright 2015 Freescale Semiconductor, Inc.
 * Copyright 2015-2021 NXP.
 *
 * License: LA_OPT_NXP_Software_License
 *
 * This software is owned or controlled by NXP and may
 * only be used strictly in accordance with the applicable license terms.
 * By expressly accepting such terms or by downloading, installing,
 * activating and/or otherwise using the software, you are agreeing that
 * you have read, and that you agree to comply with and are bound by,
 * such license terms.  If you do not agree to be bound by the applicable
 * license terms, then you may not retain, install, activate or otherwise
 * use the software.
 * 
 * @version   4.1
 * 
 * @brief     IEC60730 Class B Safety Routines for Digital I/O dedicated for MC65F83xxx, MC56F81xxx
 *
 */
#ifndef _IEC60730B_DIO_
#define _IEC60730B_DIO_

/******************************************************************************
 * Macros
 ******************************************************************************/
/* DIO test result constants */
#define FS_DIO_PASS  (0x0)
#define FS_DIO_FAIL  (FS_DIO_FAIL_CODE|0x1)


#ifndef __ASM__

/*******************************************************************************
* Definitions
******************************************************************************/
/* Used to backup original DIO port/pin configuration */
typedef struct
{
	uint16_t PUR; /* GPIO Pull Resistor Enable Register */
	uint16_t PUS; /* GPIO Pull Resistor Type Select */
	uint16_t DDR; /* GPIO Data Direction Register */
	uint16_t DR;  /* GPIO Data Register */
} fs_dio_backup_t;


/* Safety DIO test item */
typedef struct
{
	uint16_t gpio;   /* Pointer to GPIO module  */
	uint8_t  pinNum;
	uint8_t  pinDir;
	uint8_t  pinMux;
    fs_dio_backup_t sTestedPinBackup;
} fs_dio_test_t;



/******************************************************************************
* Functions
******************************************************************************/
/*******************************************************************************
*
* @brief  This function executes the digital output test. The test is focused only to test one digital pin. 
*
* @param  *pTestedPin - pointer to tested pin struct (fs_dio_test_t)
* @param  delay       - the delay needed to recognise the value change on tested pin
*
* @return - FS_DIO_FAIL        
*         - FS_DIO_PASS
*
*******************************************************************************/
FS_RESULT FS_DIO_Output(fs_dio_test_t *pTestedPin, uint16_t delay);

#endif /* __ASM__ */

#endif /* _IEC60730B_DIO_ */
