/*
 * Copyright 2015 Freescale Semiconductor, Inc.
 * Copyright 2015-2021 NXP.
 *
 * License: LA_OPT_NXP_Software_License
 *
 * This software is owned or controlled by NXP and may
 * only be used strictly in accordance with the applicable license terms.
 * By expressly accepting such terms or by downloading, installing,
 * activating and/or otherwise using the software, you are agreeing that
 * you have read, and that you agree to comply with and are bound by,
 * such license terms.  If you do not agree to be bound by the applicable
 * license terms, then you may not retain, install, activate or otherwise
 * use the software.
 * 
 * @version   4.1
 * 
 * @brief     IEC60730 Class B Safety Routines for the watchdog test dedicated for MC65F83xxx, MC56F81xxx
 *
 */

#include "iec60730b.h"
/*******************************************************************************
 * Definitions
 ******************************************************************************/
typedef struct
{
	unsigned short COP_CTRL;
	unsigned short COP_TOUT;
	unsigned short COP_CNTR;
	unsigned short COP_INTVAL;
	unsigned short COP_WINDOW;
} FS_WDOG_Type;


#define Watchdog_BASE     (0xE320)
#define WATCHDOG_REG      ((FS_WDOG_Type   *)  Watchdog_BASE)


typedef struct
{
	unsigned short PIT_CTRL;
	unsigned short PIT_MOD;
	unsigned short PIT_CNTR;
}FS_PIT_Type;

typedef struct
{
	unsigned short PIT_CTRL;
	unsigned short PIT_MOD_L;
	unsigned short PIT_MOD_H;
	unsigned short PIT_CNTR_L;
	unsigned short PIT_CNTR_H;
}FS_PIT_32_Type;


#define SIM_RSTAT_BASE    0xE401
#define SIM_RSTAT *(unsigned short*) (SIM_RSTAT_BASE)
#define COP_CPU 0x20 		/* COP CPU Time-out Reset */

/*******************************************************************************
 * Implementation
 ******************************************************************************/
/*******************************************************************************
 ******************************************************************************/
void FS_watchdog_setup(fs_wdog_test* pWatchdogBackup, unsigned short pit_base, unsigned char counter_32_bit)
{
    unsigned long counter_aux = 0;
	pWatchdogBackup->resets=0;                    /* clear the number of WD resets */
    pWatchdogBackup->wd_test_uncomplete_flag =0;  /* clear the wd_test_uncomplete flag */
    /* watchdog refresh */
    WATCHDOG_REG->COP_CNTR = 0x5555;
    WATCHDOG_REG->COP_CNTR = 0xAAAA;
    
    if(counter_32_bit)
    {
        while(1)
        {       	
        	counter_aux = (unsigned long)(((FS_PIT_32_Type *) pit_base)->PIT_CNTR_H) << 16U;
        	counter_aux |= (unsigned long)((FS_PIT_32_Type *) pit_base)->PIT_CNTR_L;
        	pWatchdogBackup->counter = counter_aux;
        }   	    	
    }
    else 
    {
        while(1)
        {
        	pWatchdogBackup->counter = ((FS_PIT_Type *) pit_base)->PIT_CNTR;        	        	
        }      	
    }
}


/*******************************************************************************
 ******************************************************************************/
void FS_watchdog_check(unsigned long limit_high, unsigned long limit_low, unsigned short resets_limit, unsigned short endless_loop_enable, fs_wdog_test* pWatchdogBackup)
{
    unsigned short error_flag = 0;
    /* if COP reset has occured */
    if((SIM_RSTAT & COP_CPU) == COP_CPU) 
    {
        pWatchdogBackup->resets +=1; /* increment the number of WD resets */
        /* check if the watchdog resets the system too many times */
        if (pWatchdogBackup->resets >= resets_limit)
        {
            pWatchdogBackup->resets = resets_limit; /* to avoid overflow of reset counter */
            error_flag = 1;   /* set the error flag */
        }
        /* check if the timeout from watchdog test corresponds to value of independent timer */
        if ((pWatchdogBackup->counter > limit_high)||(pWatchdogBackup->counter < limit_low))
        {
            error_flag = 1;   /* set the error flag */
        }
    }
    else 
    {
        pWatchdogBackup->wd_test_uncomplete_flag = 1;     /* set the flag to 1 */
        error_flag = 1;   /* set the error flag */
    }
    /* if endless loop is enabled and some error has occured, wait in loop */
    if (error_flag && endless_loop_enable)
    {
      while(1)
      {
      }
    }    
}

